/*********************************************************************
 *
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2008, Willow Garage, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Willow Garage nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *  \author Joe Romano
 *********************************************************************/

#include <pr2_controller_interface/controller.h>
#include <pr2_hardware_interface/hardware_interface.h>

#include <slipgrip_controller/pressureObserver.h>
#include <slipgrip_controller/accelerationObserver.h>

#include <slipgrip_controller/AccelerationSensorData.h>
#include <slipgrip_controller/GripperSensorData.h>
#include <slipgrip_controller/PR2GripperSensorData.h>
#include <pr2_controllers_msgs/JointControllerState.h>

#include <std_srvs/Empty.h>
#include <std_msgs/Empty.h>

#include <ros/ros.h>
#include <rosrt/rosrt.h>

#include <boost/shared_ptr.hpp>
#include <boost/thread/condition.hpp>
#include <realtime_tools/realtime_box.h>
#include <realtime_tools/realtime_publisher.h>
#include <std_msgs/Float64.h>

namespace slipgrip_controller{
 
class PR2GripperSensorObserver: public pr2_controller_interface::Controller
{
private:
  // our ROS node handle
  ros::NodeHandle node_handle_;
  // our robot state handle
  pr2_mechanism_model::RobotState *robot_handle_;

  // functions
  bool initializeHandles(pr2_mechanism_model::RobotState *robot, ros::NodeHandle &n);

  ros::Time last_time_;

  // handles to our sensors and joint
  pr2_hardware_interface::Accelerometer* accelerometer_handle_;
  AccelerationSensorData acc_data_;

  // accelerometer observer
  accelerationObserver *acceleration_observer_;

  // pressure observer object to do analysis on the incoming pressure sensor data
  pressureObserver *pressure_observer_;
  
  ros::ServiceServer update_zeros_srv_;

  // blank request/responses
  std_srvs::Empty::Request empty_req_;
  std_srvs::Empty::Response empty_resp_;

  //our function definitions for the services we will provide
  bool updateZeros(std_srvs::Empty::Request& req, std_srvs::Empty::Response& resp);
  void reinitializeValues();

  // tactile sensor info real-time publisher
  boost::shared_ptr<rosrt::Publisher<slipgrip_controller::PR2GripperSensorData> > tactile_state_publisher_;

  void publish();
  int publishing_counter_;
  int loop_counter_;

  double zeroing_duration_;
  ros::Time zeroing_time_;
  int mode_;
  bool zeroing_;
  
public:
  PR2GripperSensorObserver();
  ~PR2GripperSensorObserver();
  virtual bool init(pr2_mechanism_model::RobotState *robot, ros::NodeHandle &n);
  virtual void update();
  virtual void stopping();
  virtual void starting() {}
};
} 
