function [] = showAccPdP(myfilename)


% load up a file we like
[pressure_left pressure_right pressureTime] = loadAndProcessGripperSensorRTP(myfilename);


% subsample down to the actual data update rate (41 ms sampling time)
pressure_downsample_coefficients = 41;
[ pressure_subsampled_left pressure_subsampled_time_left] = subsamplePressureVector(pressure_left, pressureTime,pressure_downsample_coefficients);
[ pressure_subsampled_right pressure_subsampled_time_right] = subsamplePressureVector(pressure_right, pressureTime,pressure_downsample_coefficients);



% threshold all our pressure streams with an adaptive filter 
[ pressure_left_thresh ] = pressureThreshVector(pressure_subsampled_left, 0.0, true);
[ pressure_right_thresh ] =  pressureThreshVector(pressure_subsampled_right, 0.0, true);


% zero out all the pressures based on the first 25 samples (1 second at 24.9x hz)
% if 1khz recording rate
[ pressure_left_zerod ] = zeroPressureVector(pressure_left_thresh, 25);
[ pressure_right_zerod ] = zeroPressureVector(pressure_right_thresh, 25);



% create a summed pressure vector over all taxels
[ lpressure_vector_sum] = sumPressureVector(pressure_left_zerod);
[ rpressure_vector_sum] = sumPressureVector(pressure_right_zerod);



% create the first subplot (Acceleration)
%subplot(5,1,1);
%plot(accTime,acc_combined(:,1)+acc_combined(:,2)+acc_combined(:,3));
%xlabel('Time (s)');
%title('Acceleration Sum');



% create the second subplot (Pressure)
subplot(4,1,1);
plot(pressure_subsampled_time_left,lpressure_vector_sum,'r',pressure_subsampled_time_right,rpressure_vector_sum,'b');
ylabel('Pressure (cnt)');
title('Total Pressure (Sum of all FingerPad cells): \SigmaP_{x,y}');
legend('Left Finger','Right Finger');

%create a plot of the derivative of our summed pressure vector
subplot(4,1,2);
lpressure_sum_deriv = [0; abs(diff(lpressure_vector_sum)/(pressure_downsample_coefficients/1000.0))];
rpressure_sum_deriv = [0; abs(diff(rpressure_vector_sum)/(pressure_downsample_coefficients/1000.0))];
plot(pressure_subsampled_time_left,lpressure_sum_deriv,'r',pressure_subsampled_time_right,rpressure_sum_deriv,'b');
ylabel('dPressure/dt (cnt)');
title('Total Loading ( Rate of Total Pressure Change): |\Delta(\SigmaP_{x,y})|');
legend('Left Finger','Right Finger');
xl = xlim;
yl = ylim;

% create a plot of the sum of the absolute value of the derivatives of our pressure vectors
subplot(4,1,3);
rpressure_deriv_sum = zeros(length(pressure_subsampled_right(:,1)),1);  % create zero vector of the appropriate length
lpressure_deriv_sum = zeros(length(pressure_subsampled_left(:,1)),1);   % create zero vector of teh appripriate length
for i=1:length(pressure_subsampled_right(1,:))
    rpressure_deriv_sum = rpressure_deriv_sum +  [0; abs(diff(pressure_subsampled_right(:,i))/(pressure_downsample_coefficients/1000.0))];
end
for i=1:length(pressure_subsampled_left(1,:))
    lpressure_deriv_sum = lpressure_deriv_sum +  [0; abs(diff(pressure_subsampled_left(:,i))/(pressure_downsample_coefficients/1000.0))];
end
plot(pressure_subsampled_time_left,lpressure_deriv_sum,'r',pressure_subsampled_time_right,rpressure_deriv_sum,'b');
ylabel('Time (s)');
title('Total Cell Loading: \Sigma|(\DeltaP_{x,y})|');
legend('Left Finger','Right Finger');
axis([xl yl]);


% create a plot that is the difference of the above two (with abs on the
% 3rd plot)
subplot(4,1,4);
plot(pressure_subsampled_time_left,lpressure_deriv_sum-lpressure_sum_deriv,'r',pressure_subsampled_time_right,rpressure_deriv_sum-rpressure_sum_deriv,'b');
%plot(pressure_subsampled_time_left,lpressure_deriv_sum-abs(lpressure_sum_deriv),'r');
xlabel('Time (s)');
title('Rate of Contact Location Transition (Total Cell Loading - Total Loading): plot 4- plot 3');
legend('Left Finger','Right Finger');
axis([xl yl]);

