/*********************************************************************
 *
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2008, Willow Garage, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Willow Garage nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *  \author Joe Romano
 *********************************************************************/
//@author  Joe Romano
//@email   joeromano@gmail.com
//@brief   tactile_sensor_observer.cpp

#include <slipgrip_controller/accelerationObserver.h>

// method to be called to check if we saw  a contact acceleration
bool accelerationObserver::checkPlaceContact(double dAcc)
{
  if(( fabs(aX) > dAcc) || (fabs(aY) > dAcc)  || (fabs(aZ) > dAcc)|| placeContact )
    placeContact = true;
  else
    placeContact = false;
  return placeContact;
}

// method to be called each 1ms timestep in realtime to ensure data collection and manipulation happens properly
void accelerationObserver::spin()
{
  // retrieve and filter our acceleration data
  std::vector<geometry_msgs::Vector3> threeAccs = accHandle->state_.samples_;
  for( uint  i = 0; i < threeAccs.size(); i++ )  
  {
    aX = accLPFilt[0]->getNextFilteredValue(threeAccs[i].x);
    aY = accLPFilt[1]->getNextFilteredValue(threeAccs[i].y);
    aZ = accLPFilt[2]->getNextFilteredValue(threeAccs[i].z);
    readingTime = ros::Time::now().toSec();
  }    

}

// ctor
accelerationObserver::accelerationObserver(pr2_hardware_interface::Accelerometer* accelerometerHandle)
{
  accHandle = accelerometerHandle;

  // place contact flag
  placeContact = false;

  // instatiate our acceleration values
  aX = 0;
  aY = 0;
  aZ = 0;

  // make sure the accelerometer has a 1.5 khz bandwidth          
  accHandle->command_.bandwidth_ = 6;

  // make sure the accelerometer has a +/- 2g range           
  accHandle->command_.range_ = 0;

  
  // create our filter for high-passed accelerometer data
  // 1st order butterworth. high-pass 50 hz
  float b_vfilt[] = { 0.9502, -0.9502};
  float a_vfilt[] = {1.0, -0.9004};
  for(int i=0; i < 3; i++)
  {
    accLPFilt[i] = new digitalFilter(1, true,b_vfilt,a_vfilt);
  }
  
}


// destructor
accelerationObserver::~accelerationObserver()
{
}

